import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import sys
import numpy as np

from ansurr import adjust_text as at

def plot_scores(output_dir='',cyrange=False):
	data = {}
	for line in open(output_dir+'scores.out','r'):
		if line[:4] == 'PDB:':
			line = line.split()
			pdb = line[1][:[pos for pos, char in enumerate(line[1]) if char == '_'][-1]] # accounts for "_" appearing in structure name
			model = int(line[1].split('_')[-1])
			shifts = line[3]
			shift_perc = int(line[5])
			corr_score = float(line[9])
			rmsd_score = float(line[13])
			if cyrange:
				corr_score_wd = float(line[17])
				rmsd_score_wd = float(line[21])
		
			if line[9] != 'nan':
				if pdb not in data:
					data[pdb] = {}
		
				if shifts not in data[pdb]:
					if cyrange:
						if str(corr_score_wd) != 'nan':
							data[pdb][shifts] = {'model':[model],'rmsd':[rmsd_score],'corr':[corr_score],'rmsd_welldefined':[rmsd_score_wd],'corr_welldefined':[corr_score_wd],'shift_perc':[shift_perc]}
					else:
						if str(corr_score) != 'nan':
							data[pdb][shifts] = {'model':[model],'rmsd':[rmsd_score],'corr':[corr_score],'shift_perc':[shift_perc]}

				else:
					if str(corr_score) != 'nan':
						data[pdb][shifts]['model'].append(model)
						data[pdb][shifts]['rmsd'].append(rmsd_score)
						data[pdb][shifts]['corr'].append(corr_score)
						data[pdb][shifts]['shift_perc'].append(shift_perc)
					if cyrange:
						if str(corr_score_wd) != 'nan':
							data[pdb][shifts]['rmsd_welldefined'].append(rmsd_score_wd)
							data[pdb][shifts]['corr_welldefined'].append(corr_score_wd)

				
	for pdb in data:
		for shifts in data[pdb]:
			plt.cla()
			plt.scatter(data[pdb][shifts]['rmsd'], data[pdb][shifts]['corr'], c='black',s=8)

			labels = [plt.text(data[pdb][shifts]['rmsd'][i], data[pdb][shifts]['corr'][i], data[pdb][shifts]['model'][i],
							   ha='center', va='center', color='blue',size=6) for i in range(len(data[pdb][shifts]['rmsd']))]

			plt.ylabel('correlation score',size=12)
			plt.xlabel('RMSD score',size=12)
			plt.axis('scaled')  
			plt.xlim(-5,105)
			plt.ylim(-5,105)
			shift_perc = int(round(np.nanmean(data[pdb][shifts]['shift_perc']),0)) # just in case models don't have the same shift completeness - very unlikely
			if shift_perc < 75:
				shift_perc_out = str(shift_perc) + ' (unreliable!)'
			else:
				shift_perc_out = str(shift_perc)

			plt.title('Structure: '+pdb+'\nShifts: '+shifts+' Shift%: '+shift_perc_out)
			at.adjust_text(labels,expand_text=(1.75, 1.75), expand_points=(1.75, 1.75),arrowprops=dict(arrowstyle='->', color='red', alpha=0.6, linewidth=0.5))
			plt.tight_layout()
			plt.savefig(output_dir+pdb+'_'+shifts+'.png',dpi=300)


			if cyrange:
				plt.cla()
				plt.scatter(data[pdb][shifts]['rmsd_welldefined'], data[pdb][shifts]['corr_welldefined'], c='black',s=8)
				labels_cyrange = [plt.text(data[pdb][shifts]['rmsd_welldefined'][i], data[pdb][shifts]['corr_welldefined'][i], data[pdb][shifts]['model'][i],
							   ha='center', va='center', color='blue',size=6) for i in range(len(data[pdb][shifts]['rmsd_welldefined']))]

				plt.ylabel('correlation score',size=12)
				plt.xlabel('RMSD score',size=12)
				plt.axis('scaled')  
				plt.xlim(-5,105)
				plt.ylim(-5,105)
				shift_perc = int(round(np.nanmean(data[pdb][shifts]['shift_perc']),0)) # have sep shift completness for well defined region?
				if shift_perc < 75:
					shift_perc_out = str(shift_perc) + ' (unreliable!)'
				else:
					shift_perc_out = str(shift_perc)

				plt.title('Structure: '+pdb+' (WD)\nShifts: '+shifts+' Shift%: '+shift_perc_out)
				at.adjust_text(labels_cyrange,expand_text=(1.75, 1.75), expand_points=(1.75, 1.75),arrowprops=dict(arrowstyle='->', color='red', alpha=0.6, linewidth=0.5))
				plt.tight_layout()
				plt.savefig(output_dir+pdb+'_'+shifts+'_welldefined.png',dpi=300)

def main():

    plot_scores(sys.argv[1])
    sys.exit(0)
   
if __name__ == "__main__":
    main()


#!/usr/bin/env python

import sys
import os
import json
import subprocess

from sys import platform
from importlib import resources

from ansurr.functions import check_quiet_print

def calc_welldefined(pdb,output_dir='',quiet=False):

    if platform == "linux" or platform == "linux2":
        with resources.path("ansurr.bin", "cyrange.bin") as f:  # use this for the package
        #with resources.path("bin", "cyrange.bin") as f: 
            cyrange_file_path = f
    elif platform == "darwin":
        with resources.path("ansurr.bin", "cyrange_OSX.bin") as f:  # use this for the package
        #with resources.path("bin", "cyrange.bin") as f: 
            cyrange_file_path = f

    cyrange_output = subprocess.run([cyrange_file_path, pdb],stdout=subprocess.PIPE, universal_newlines = True, stderr=open(os.devnull, 'w'))

    cyrange = {'default chain':[]}

    for cyrange_out in cyrange_output.stdout.split('\n'):
        if "Optimal range" in cyrange_out:
            cyrange_out = cyrange_out[18:]
            gaps = int(cyrange_out.split(':')[1].split(',')[1].split()[0])
            if gaps == 0:
                if cyrange_out[0].isalpha():
                    chain = cyrange_out[0]
                    first_resi = int(cyrange_out.split(':')[0].split('..')[0][1:])
                    last_resi = int(cyrange_out.split(':')[0].split('..')[1][1:])
                    if chain not in cyrange:
                        cyrange[chain] = []
                    cyrange[chain].extend(list(range(first_resi,last_resi+1)))
                else:
                    first_resi = int(cyrange_out.split(':')[0].split('..')[0])
                    last_resi = int(cyrange_out.split(':')[0].split('..')[1])
                    cyrange['default chain'].extend(list(range(first_resi,last_resi+1)))
            else:
                for r in cyrange_out.split(':')[0].split(','):
                    r = r.strip()
                    if '..' in r:
                        if r[0].isalpha():
                            chain = r[0]
                            first_resi = int(r.split(':')[0].split('..')[0][1:])
                            last_resi = int(r.split(':')[0].split('..')[1][1:])
                            if chain not in cyrange:
                                cyrange[chain] = []
                            cyrange[chain].extend(list(range(first_resi,last_resi+1)))
                        else:
                            first_resi = int(r.split(':')[0].split('..')[0])
                            last_resi = int(r.split(':')[0].split('..')[1])
                            cyrange['default chain'].extend(list(range(first_resi,last_resi+1)))
                    else:
                        if r[0].isalpha():
                            chain = r[0]
                            cyrange[chain].extend([int(r)])
                        else:
                            cyrange['default chain'].extend([int(r)])

    chain_resi = {}
    for chain in cyrange:
        resi = len(cyrange[chain])
        if resi > 0:
            chain_resi[chain] = resi
      
    if len(chain_resi) > 0: 
        if len(chain_resi) == 1:
            check_quiet_print(quiet,' -> identified '+str(chain_resi['default chain'])+' well-defined residues')
        else:
            msg=' -> identified'
            for chain in chain_resi:
                msg+=' '+str(chain_resi[chain])+' ('+chain+'),'
            msg=msg[:-1]+" well-defined residues"
            print(msg)
        
        if not os.path.exists(output_dir+'/other_output/cyrange/'):
            os.makedirs(output_dir+'/other_output/cyrange/')
        with open(output_dir+'/other_output/cyrange/cyrange.json', 'w') as outfile:
            json.dump(cyrange, outfile)
        return True

    else:
        check_quiet_print(quiet," -> ERROR CYRANGE failed to identify well-defined residues")
        return False

def main():

    cyrange(sys.argv[1])
    sys.exit(0)
   
if __name__ == "__main__":
    main()


import sys

import pynmrstar

#from ansurr.functions import check_quiet_print

def export(nef_output, output_dir='',ansurr_version='.',pdb_file='.',shift_file='.',reref='.',lig='.',nonstd='.',olig='.',quiet=False):

    ansurr_per_model_lp = pynmrstar.Loop.from_scratch()
    ansurr_per_model_lp.add_tag(['_ansurr_per_model.model_code','_ansurr_per_model.pdb_chain_code','_ansurr_per_model.shifts_set_code','_ansurr_per_model.shifts_chain_code','_ansurr_per_model.backbone_shift_completeness', '_ansurr_per_model.correlation', '_ansurr_per_model.correlation_score','_ansurr_per_model.rmsd','_ansurr_per_model.rmsd_score', '_ansurr_per_model.correlation_well_defined', '_ansurr_per_model.correlation_score_well_defined','_ansurr_per_model.rmsd_well_defined','_ansurr_per_model.rmsd_score_well_defined'])

    for chain in nef_output['per_model']:
        for rci_set in nef_output['per_model'][chain]:
            for rci_chain in nef_output['per_model'][chain][rci_set]:
                for model in nef_output['per_model'][chain][rci_set][rci_chain]:
                    ansurr_per_model_lp.add_data([model, chain, rci_set, rci_chain,nef_output['per_model'][chain][rci_set][rci_chain][model]['shift_completeness'], nef_output['per_model'][chain][rci_set][rci_chain][model]['corr'],nef_output['per_model'][chain][rci_set][rci_chain][model]['corr_score'],nef_output['per_model'][chain][rci_set][rci_chain][model]['rmsd'],nef_output['per_model'][chain][rci_set][rci_chain][model]['rmsd_score'],nef_output['per_model'][chain][rci_set][rci_chain][model]['corr_wd'],nef_output['per_model'][chain][rci_set][rci_chain][model]['corr_score_wd'],nef_output['per_model'][chain][rci_set][rci_chain][model]['rmsd_wd'],nef_output['per_model'][chain][rci_set][rci_chain][model]['rmsd_score_wd']])


    #print(ansurr_per_model_lp)

    ansurr_per_residue_lp = pynmrstar.Loop.from_scratch()
    ansurr_per_residue_lp.add_tag(['_ansurr_per_residue.model_code','_ansurr_per_residue.chain_code','_ansurr_per_residue.shifts_set_code','_ansurr_per_residue.shifts_chain_code','_ansurr_per_residue.sequence_code', '_ansurr_per_residue.residue_name', '_ansurr_per_residue.rci', '_ansurr_per_residue.rigidity','_ansurr_per_residue.secondary_structure','_ansurr_per_residue.welldefined','_ansurr_per_residue.shift_types'])

    for chain in nef_output['per_residue']:
        for rci_set in nef_output['per_residue'][chain]:
            for rci_chain in nef_output['per_residue'][chain][rci_set]:
                for model in nef_output['per_residue'][chain][rci_set][rci_chain]:
                    for residue in nef_output['per_residue'][chain][rci_set][rci_chain][model]:
                        ansurr_per_residue_lp.add_data([model, chain, rci_set, rci_chain,residue, nef_output['per_residue'][chain][rci_set][rci_chain][model][residue]['resn'],nef_output['per_residue'][chain][rci_set][rci_chain][model][residue]['rci'],nef_output['per_residue'][chain][rci_set][rci_chain][model][residue]['rigidity'],nef_output['per_residue'][chain][rci_set][rci_chain][model][residue]['ss'],nef_output['per_residue'][chain][rci_set][rci_chain][model][residue]['wd'],nef_output['per_residue'][chain][rci_set][rci_chain][model][residue]['shift_types']])

    #print(ansurr_per_residue_lp)

    ansurr_sf = pynmrstar.Saveframe.from_scratch("ansurr", "ansurr")
          
            
    ansurr_sf.add_tag("sf_category", "ansurr")
    ansurr_sf.add_tag("sf_framecode", "ansurr")

    ansurr_sf.add_tag("_ansurr.ansurr_version", "ansurr")
    ansurr_sf.add_tag("_ansurr.panav_version", "ansurr")
    ansurr_sf.add_tag("_ansurr.dssp_version", "ansurr")
    ansurr_sf.add_tag("_ansurr.cyrange_version", "ansurr")

    ansurr_sf.add_tag("_ansurr.input_pdb_file", "ansurr")
    ansurr_sf.add_tag("_ansurr.input_shift_file", "ansurr")
    ansurr_sf.add_tag("_ansurr.rereference_shifts", "ansurr")
    ansurr_sf.add_tag("_ansurr.include_ligands", "ansurr")
    ansurr_sf.add_tag("_ansurr.include_nonstd_res", "ansurr")
    ansurr_sf.add_tag("_ansurr.oligomer", "ansurr")

    ansurr_sf['ansurr_version'] = ansurr_version
    ansurr_sf['panav_version'] = "2.1"
    ansurr_sf['dssp_version'] = "3.0.0"
    ansurr_sf['cyrange_version'] = "2.0"


    ansurr_sf['input_pdb_file'] = pdb_file
    ansurr_sf['input_shift_file'] = shift_file

    ansurr_sf['rereference_shifts'] = reref
    ansurr_sf['include_ligands'] = lig
    ansurr_sf['include_nonstd_res'] = nonstd
    ansurr_sf['oligomer'] = olig

    # Now add the loop we created before
    ansurr_sf.add_loop(ansurr_per_model_lp)
    ansurr_sf.add_loop(ansurr_per_residue_lp)
    
    # Now write out our saveframe to a file. Optionally specify format="json" to write in JSON format.
    ansurr_sf.write_to_file(output_dir+"_ansurr.nef")
    ansurr_sf.write_to_file(output_dir+"_ansurr.json", format_="json")


def main():
    sys.exit(0)
   
if __name__ == "__main__":
    main()